From: Andrew Morton <akpm@linux-foundation.org>
To: Christoph Lameter <cl@linux.com>
Cc: Xi Wang <xi.wang@gmail.com>,
Dan Carpenter <dan.carpenter@oracle.com>,
Jesper Juhl <jj@chaosbits.net>, Jens Axboe <axboe@kernel.dk>,
Pekka Enberg <penberg@kernel.org>,
linux-kernel@vger.kernel.org, Matt Mackall <mpm@selenic.com>,
David Rientjes <rientjes@google.com>
Subject: Re: Uninline kcalloc
Date: Tue, 14 Feb 2012 12:45:18 -0800 [thread overview]
Message-ID: <20120214124518.f42bc03e.akpm@linux-foundation.org> (raw)
In-Reply-To: <alpine.DEB.2.00.1202141329360.24883@router.home>
On Tue, 14 Feb 2012 13:33:40 -0600 (CST)
Christoph Lameter <cl@linux.com> wrote:
> Subject: Uninline kcalloc
>
> kcalloc is not used in performance critical ways. So it does not need to
> be inline. If we would add diagnostics to track the overflow occurrences
> then such code would be replicated at all call sites in the kernel.
Uninlining kcalloc() seems reasonable. But if we're going to uninline
kcalloc() then we also should uninline kmalloc_array().
And yes, it's still called kmalloc_array() in my tree. I've been
following this discussion for N days waiting for a reason for changing
the original patch and I ain't seen one yet.
From: Xi Wang <xi.wang@gmail.com>
Subject: slab: introduce kmalloc_array()
Introduce a kmalloc_array() wrapper that performs integer overflow
checking without zeroing the memory.
Suggested-by: Andrew Morton <akpm@linux-foundation.org>
Suggested-by: Jens Axboe <axboe@kernel.dk>
Signed-off-by: Xi Wang <xi.wang@gmail.com>
Cc: Pekka Enberg <penberg@kernel.org>
Cc: Dan Carpenter <dan.carpenter@oracle.com>
Signed-off-by: Andrew Morton <akpm@linux-foundation.org>
---
include/linux/slab.h | 17 ++++++++++++++---
1 file changed, 14 insertions(+), 3 deletions(-)
diff -puN include/linux/slab.h~slab-introduce-kmalloc_array include/linux/slab.h
--- a/include/linux/slab.h~slab-introduce-kmalloc_array
+++ a/include/linux/slab.h
@@ -190,7 +190,7 @@ size_t ksize(const void *);
#endif
/**
* kcalloc - allocate memory for an array. The memory is set to zero.
* @n: number of elements.
* @size: element size.
* @flags: the type of memory to allocate.
@@ -240,11 +240,22 @@ size_t ksize(const void *);
* for general use, and so are not documented here. For a full list of
* potential flags, always refer to linux/gfp.h.
*/
-static inline void *kcalloc(size_t n, size_t size, gfp_t flags)
+static inline void *kmalloc_array(size_t n, size_t size, gfp_t flags)
{
if (size != 0 && n > ULONG_MAX / size)
return NULL;
- return __kmalloc(n * size, flags | __GFP_ZERO);
+ return __kmalloc(n * size, flags);
+}
+
+/**
+ * kcalloc - allocate memory for an array. The memory is set to zero.
+ * @n: number of elements.
+ * @size: element size.
+ * @flags: the type of memory to allocate (see kmalloc).
+ */
+static inline void *kcalloc(size_t n, size_t size, gfp_t flags)
+{
+ return kmalloc_array(n, size, flags | __GFP_ZERO);
}
#if !defined(CONFIG_NUMA) && !defined(CONFIG_SLOB)
_
next prev parent reply other threads:[~2012-02-14 20:45 UTC|newest]
Thread overview: 57+ messages / expand[flat|nested] mbox.gz Atom feed top
2012-02-07 14:11 integer overflows in kernel/relay.c Dan Carpenter
2012-02-08 8:34 ` Jens Axboe
2012-02-08 22:25 ` Andrew Morton
2012-02-09 12:41 ` Jens Axboe
2012-02-09 17:39 ` Andrew Morton
2012-02-09 12:41 ` [PATCH RFC] slab: introduce knalloc/kxnalloc Xi Wang
2012-02-09 13:05 ` Pekka Enberg
2012-02-09 13:19 ` Jens Axboe
2012-02-09 13:26 ` Xi Wang
2012-02-09 13:48 ` [PATCH RFC v2] slab: introduce kmalloc_array Xi Wang
2012-02-09 22:42 ` David Rientjes
2012-02-09 23:08 ` Andrew Morton
2012-02-09 22:47 ` Jesper Juhl
2012-02-09 23:06 ` Andrew Morton
2012-02-09 23:43 ` Joe Perches
2012-02-13 15:08 ` Xi Wang
2012-02-13 16:01 ` Christoph Lameter
2012-02-13 19:44 ` Dan Carpenter
2012-02-13 20:27 ` Christoph Lameter
2012-02-14 7:20 ` Dan Carpenter
2012-02-14 7:35 ` Pekka Enberg
2012-02-14 11:12 ` Xi Wang
2012-02-14 15:02 ` Christoph Lameter
2012-02-14 16:30 ` Xi Wang
2012-02-14 16:34 ` Christoph Lameter
2012-02-14 16:43 ` Xi Wang
2012-02-14 19:33 ` Uninline kcalloc Christoph Lameter
2012-02-14 19:37 ` Christoph Lameter
2012-02-14 20:46 ` Andrew Morton
2012-02-14 20:50 ` Nick Bowler
2012-02-14 21:24 ` Christoph Lameter
2012-02-15 20:17 ` Nick Bowler
2012-02-15 20:24 ` Nick Bowler
2012-02-14 20:45 ` Andrew Morton [this message]
2012-02-14 20:58 ` Pekka Enberg
2012-02-14 21:09 ` Christoph Lameter
2012-02-14 21:31 ` Pekka Enberg
2012-02-14 21:38 ` Christoph Lameter
2012-02-14 21:46 ` Xi Wang
2012-02-14 22:08 ` Christoph Lameter
2012-02-15 19:14 ` Xi Wang
2012-02-15 19:34 ` Christoph Lameter
2012-02-16 3:10 ` Xi Wang
2012-02-16 14:51 ` Christoph Lameter
2012-02-16 18:32 ` Xi Wang
2012-02-16 20:47 ` Christoph Lameter
2012-02-10 13:09 ` [PATCH RFC v2] slab: introduce kmalloc_array Alexey Dobriyan
2012-02-10 13:11 ` Alexey Dobriyan
2012-02-10 13:55 ` Xi Wang
2012-02-10 13:58 ` Alexey Dobriyan
2012-02-10 14:09 ` Xi Wang
2012-02-11 12:19 ` Alexey Dobriyan
2012-02-12 5:46 ` Xi Wang
2012-02-09 12:56 ` integer overflows in kernel/relay.c Pekka Enberg
2012-02-09 10:44 ` [patch] relay: prevent integer overflow in relay_open() Dan Carpenter
2012-02-09 11:55 ` walter harms
2012-02-09 12:36 ` Dan Carpenter
Reply instructions:
You may reply publicly to this message via plain-text email
using any one of the following methods:
* Save the following mbox file, import it into your mail client,
and reply-to-all from there: mbox
Avoid top-posting and favor interleaved quoting:
https://en.wikipedia.org/wiki/Posting_style#Interleaved_style
* Reply using the --to, --cc, and --in-reply-to
switches of git-send-email(1):
git send-email \
--in-reply-to=20120214124518.f42bc03e.akpm@linux-foundation.org \
--to=akpm@linux-foundation.org \
--cc=axboe@kernel.dk \
--cc=cl@linux.com \
--cc=dan.carpenter@oracle.com \
--cc=jj@chaosbits.net \
--cc=linux-kernel@vger.kernel.org \
--cc=mpm@selenic.com \
--cc=penberg@kernel.org \
--cc=rientjes@google.com \
--cc=xi.wang@gmail.com \
/path/to/YOUR_REPLY
https://kernel.org/pub/software/scm/git/docs/git-send-email.html
* If your mail client supports setting the In-Reply-To header
@@ -190,7 +190,7 @@ size_t ksize(const void *);
#endif
/**
- * kcalloc - allocate memory for an array. The memory is set to zero.
+ * kmalloc_array - allocate memory for an array.
* @n: number of elements.
* @size: element size.
via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line
before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox
Powered by JetHome